'''
Created on 09/07/2014

@author: synerty
'''

# from peek_server.server.live_db.LiveDb import liveDb
# from peek_server.server.orm.LiveDb import LiveDbKey
# from peek_server.server.orm.AgentData import AgentImportDispGridInfo
# from peek_server.server.orm.Display import DispBase
# from peek_server.server.orm.GridKeyIndex import GridKeyCompilerQueue, DispIndexerQueue
from txhttputil.util.DeferUtil import deferToThreadWrap
from vortex.handler.ModelHandler import ModelHandler

executeListDataKey = {'plugin': 'peek_server',
                      'key': "peakadm.dashboard.list.data"}


class DashboardListHandler(ModelHandler):

    @deferToThreadWrap
    def buildModel(self, payloadFilt, **kwargs):
        qryItems = []

        from peek_server.storage import dbConn
        session = dbConn.ormSessionCreator()

        def rowsQuick(Declarative):
            sql = '''
                  SELECT 50 * count(*) AS estimate
                  FROM "%s" TABLESAMPLE SYSTEM (2);
                  '''
            sql %= Declarative.__table__.name
            return session.execute(sql).fetchone()[0]

        def rows(Declarative, quick=False):
            if quick:
                return rowsQuick(Declarative)

            return session.query(Declarative).count()

        # qryItems.append(("Grid Compiler Queue Size", GridKeyCompilerQueue, False))
        # qryItems.append(("Display Compiler Queue Size", DispIndexerQueue, False))
        # qryItems.append(("Display Agent Imported Grids", AgentImportDispGridInfo, False))
        # qryItems.append(("Display Objects (Estimate)", DispBase, True))
        # qryItems.append(("Live DB Values (Estimate)", LiveDbKey, True))

        session.close()

        data = [dict(desc=desc, value=rows(Declarative, quick))
                for desc, Declarative, quick in qryItems]

        # from peek_server.server.queue_processesors.GridKeyQueueCompiler import gridKeyQueueCompiler
        # data.append(dict(desc="Grid Compiler Status",
        #                  value=gridKeyQueueCompiler.statusText()))
        #
        # from peek_server.server.queue_processesors.DispQueueIndexer import dispQueueCompiler
        # data.append(dict(desc="Display Compiler Status",
        #                  value=dispQueueCompiler.statusText()))

        # data.extend([dict(desc=i[0], value=i[1]) for i in liveDb.statusNameValues])

        return data


dashboardListHandler = DashboardListHandler(executeListDataKey)



import logging
import os
import shutil
import sys
import tarfile

from pytmpdir.Directory import Directory
from twisted.internet.defer import inlineCallbacks
from txhttputil.util.DeferUtil import deferToThreadWrap

from peek_platform import PeekPlatformConfig
from peek_platform.sw_install.PeekSwInstallManagerABC import PEEK_PLATFORM_STAMP_FILE, \
    PeekSwInstallManagerABC
from peek_platform.util.PtyUtil import spawnPty, logSpawnException, spawnSubprocess

__author__ = 'synerty'

logger = logging.getLogger(__name__)


class PeekSwUploadManager(object):
    def __init__(self):
        pass

    @inlineCallbacks
    def processUpdate(self, namedTempFile):

        if not tarfile.is_tarfile(namedTempFile.name):
            raise Exception("Uploaded archive is not a tar file")

        newVersion, newPath = yield self.updateToTarFile(namedTempFile.name)

        # Tell the peek server to install and restart
        yield PeekPlatformConfig.peekSwInstallManager.installAndRestart(newVersion)

        return newVersion

    @deferToThreadWrap
    def updateToTarFile(self, newSoftwareTar):
        """ Update To Tar File

        This method inspects the tar file and finally extracts it to the platform
        path.

        """

        directory = Directory()
        tarfile.open(newSoftwareTar).extractall(directory.path)
        directory.scan()

        self._testPackageUpdate(directory)

        platformVersionFile = [f for f in directory.files if
                               f.name == PEEK_PLATFORM_STAMP_FILE]
        if len(platformVersionFile) != 1:
            raise Exception("Uploaded archive does not contain a Peek Platform update"
                            ", Expected 1 %s file, got %s"
                            % (PEEK_PLATFORM_STAMP_FILE, len(platformVersionFile)))

        platformVersionFile = platformVersionFile[0]

        if '/' in platformVersionFile.path:
            raise Exception("Expected %s to be one level down, it's at %s"
                            % (PEEK_PLATFORM_STAMP_FILE, platformVersionFile.path))

        with platformVersionFile.open() as f:
            newVersion = f.read().strip()

        newPath = PeekSwInstallManagerABC.makeReleaseFileName(newVersion)

        # Do we really need to keep the old version if it's the same build?
        if os.path.isdir(newPath):
            shutil.rmtree(newPath)

        elif os.path.isfile(newPath):
            os.remove(newPath)

        shutil.copy(newSoftwareTar, newPath)

        return newVersion, newPath

    def _testPackageUpdate(self, directory: Directory) -> None:
        """ Test Package Update

        :param directory: The directory where the packages are extracted

        Since we ARE running on the server, we will test install these packages here
         first, this is done by creating a virtualenv.

        There isn't a lot of testing for the release at this stage.
        Currently we just use pip to try and install the packages off line, if it's happy
         we're happy.

        """

        # Create the test virtualenv
        virtualEnvDir = Directory()

        virtExec = os.path.join(os.path.dirname(sys.executable), "virtualenv")
        virtArgs = [virtExec,
                    # Give the virtual environment access to the global
                    '--system-site-packages',
                    virtualEnvDir.path]
        virtArgs = ' '.join(virtArgs)

        try:
            spawnSubprocess(virtArgs)
            logger.debug("Vritual env created.")

        except Exception as e:
            logSpawnException(e)
            e.message = "Failed to create virtualenv for platform test"
            raise

        # We could use import pip, pip.main(..), except:
        # We want to capture, the output, and:
        # we can't tell it to use the virtualenv

        pipExec = os.path.join(virtualEnvDir.path, 'bin', 'pip')

        # Install all the packages from the directory
        pipArgs = [pipExec] + PeekSwInstallManagerABC.makePipArgs(directory)
        pipArgs = ' '.join(pipArgs)

        try:
            spawnPty(pipArgs)

        except Exception as e:
            logSpawnException(e)

            # Update the detail of the exception and raise it
            e.message = "Test install of updated package failed."
            raise

        # Continue normally if it all succeeded
        logger.debug("Peek update successfully tested.")



"""Added file/dir name to PeekPluginInfo

Revision ID: bc1622f6c16c
Revises: c5e78990adf6
Create Date: 2016-11-05 17:58:21.169972

"""

# revision identifiers, used by Alembic.
revision = 'bc1622f6c16c'
down_revision = 'c5e78990adf6'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
import geoalchemy2


def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.add_column('PeekPluginInfo', sa.Column('dirName', sa.String(), nullable=False), schema='peek_server')
    op.add_column('PeekPluginInfo', sa.Column('fileName', sa.String(), nullable=False), schema='peek_server')
    ### end Alembic commands ###


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('PeekPluginInfo', 'fileName', schema='peek_server')
    op.drop_column('PeekPluginInfo', 'dirName', schema='peek_server')
    ### end Alembic commands ###



from peek_platform import PeekPlatformConfig
from peek_platform.sw_install.PluginSwInstallManagerABC import PluginSwInstallManagerABC


class PluginSwInstallManager(PluginSwInstallManagerABC):
    def notifyOfPluginVersionUpdate(self, pluginName, targetVersion):
        PeekPlatformConfig.pluginLoader.loadPlugin(pluginName)
        PeekPlatformConfig.pluginLoader.buildFrontend()




""" 
 * orm.Base.py
 *
 *  Copyright Synerty Pty Ltd 2013
 *
 *  This software is proprietary, you are not free to copy
 *  or redistribute this code in any format.
 *
 *  All rights to this software are reserved by 
 *  Synerty Pty Ltd
 *
"""

from sqlalchemy.ext.declarative import declarative_base

from sqlalchemy.schema import MetaData

metadata = MetaData(schema="peek_server")
DeclarativeBase = declarative_base(metadata=metadata)



from txhttputil.site.BasicResource import BasicResource

from peek_server.server.sw_download.PeekSwDownloadResource import \
    PeekSwUpdateDownloadResource
from peek_server.server.sw_download.PluginSwDownloadResource import PluginSwDownloadResource

root = BasicResource()

def setup():

    # Add the platform download resource
    root.putChild(b'peek_server.sw_install.platform.download',
                  PeekSwUpdateDownloadResource())

    # Add the plugin download resource
    root.putChild(b'peek_server.sw_install.plugin.download'
                  , PluginSwDownloadResource())




import os
from peek_server.server.sw_upload.PeekSwUploadResource import PeekSwUploadResource
from txhttputil.site.FileUnderlayResource import FileUnderlayResource

from vortex.VortexFactory import VortexFactory

root = FileUnderlayResource()

def setup():
    # Setup properties for serving the site
    root.enableSinglePageApplication()

    import peek_admin
    frontendProjectDir = os.path.dirname(peek_admin.__file__)
    buildDir = os.path.join(frontendProjectDir, 'build-web', 'dist')

    buildDirParent = os.path.dirname(buildDir)
    if not os.path.isdir(buildDirParent):
        raise NotADirectoryError(buildDirParent)

    # Make the dist dir, otherwise addFileSystemRoot throws an exception.
    # It rebuilds at a later date
    os.makedirs(buildDir, exist_ok=True)

    root.addFileSystemRoot(buildDir)

    # Create the Admin UI vortex
    from peek_platform import PeekPlatformConfig
    VortexFactory.createServer(PeekPlatformConfig.componentName, root)

    # Add the platform update upload resource
    root.putChild(b'peek_server.update.platform',
                  PeekSwUploadResource(PeekSwUploadResource.UPDATE_TYPE_PLATFORM))

    # Add the plugin update upload resource
    root.putChild(b'peek_server.update.plugin',
                  PeekSwUploadResource(PeekSwUploadResource.UPDATE_TYPE_PLUGIN))